-module(iconv).
-export([init/0, open/2, conv/2, close/1, reset/1]).

%%% op codes
-define(I_OPEN,    0).
-define(I_CONV,    1).
-define(I_CLOSE,   2).
-define(I_RESET,   3).

-define(DRV_NAME, "iconv_drv").

init() ->
	erl_ddll:start(),
	Path = code:priv_dir(iconv),
	erl_ddll:load(Path, ?DRV_NAME).

open(To, From) ->
	Port = open_port({spawn, ?DRV_NAME}, [binary]),
	To1 = list2bin(To),
	To1Len = size(To1),
	From1 = list2bin(From),
	From1Len = size(From1),
	Bin = <<?I_OPEN:8, To1Len:32/integer, To1:To1Len/binary, From1Len:32/integer, From1:From1Len/binary>>,
   	port_command(Port, [Bin]), 
	receive
		{Port, ok} -> {ok, Port};
		{Port, error, ErrMsg} -> port_close(Port), {error, ErrMsg}
	end.

close(Port) ->
	port_close(Port).

conv(Port, Data) ->
	Data1 = list2bin(Data),
	DataLen = size(Data1),
	Bin = <<?I_CONV:8, DataLen:32/integer, Data1:DataLen/binary>>,
	port_command(Port, [Bin]),
	receive
		{Port, ok, NewData} -> {ok, NewData};
		{Port, error, ErrMsg } -> {error, ErrMsg};
		{Port, error, ErrMsg, Pos, PartData} -> {error, ErrMsg, Pos, PartData}
	end.

reset(Port) ->
	Bin = <<?I_RESET:8>>,
	port_command(Port, Bin),
	receive
		{Port, ok} -> ok;
		{Port, error, ErrMsg} -> {error, ErrMsg}
	end.

list2bin(L) when is_list(L)   -> list_to_binary(L);
list2bin(B) when is_binary(B) -> B.
